package rition.backend.api.v1.collector;

import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import rition.backend.annotation.WithRequestIdResponse;
import rition.backend.annotation.paramter.RequestId;
import rition.backend.api.v1.dto.request.MetricDataUploadRequest;
import rition.backend.api.v1.dto.response.Response;
import rition.backend.service.MetricDataCollectingService;
import rition.common.data.dto.MetricDataDto;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/metric")
public class DataCollectingController {

    private final MetricDataCollectingService metricDataCollectingService;

    public DataCollectingController(MetricDataCollectingService metricDataCollectingService) {
        this.metricDataCollectingService = metricDataCollectingService;
    }

    private static final int METRIC_NUMS = 22;

    @PostMapping("/put")
    @WithRequestIdResponse
    public Response<Object> putData(@RequestBody List<MetricDataUploadRequest> uploadDataList,
                                    @RequestId String requestId) {
        // dto转换，按时间组装数据，timestamp->instanceId->metricDataDto映射
        Map<Long, Map<String, MetricDataDto>> metricDataTimestampMap = new HashMap<>(uploadDataList.size() / METRIC_NUMS);
        for (MetricDataUploadRequest uploadDataItem : uploadDataList) {

            // instanceId -> MetricDataDto
            var instanceId = uploadDataItem.getTags().get("instanceId");
            Map<String, MetricDataDto> instanceMetricDataMap =
                    metricDataTimestampMap.computeIfAbsent(
                            uploadDataItem.getTimestamp(),
                            key -> new HashMap<>()
                    );

            // metricName->metricData，多余的tag信息就不要了（毕竟用不到）
            MetricDataDto instanceMetricData = instanceMetricDataMap.computeIfAbsent(
                    instanceId,
                    id -> MetricDataDto.builder()
                            .timestamp(uploadDataItem.getTimestamp()).instanceId(id)
                            .data(new HashMap<>(METRIC_NUMS))
                            .build()
            );

            instanceMetricData.getData().put(uploadDataItem.getMetric(), uploadDataItem.getValue());
        }

        metricDataCollectingService.receiveData(requestId, metricDataTimestampMap);

        return Response.success();
    }
}
