package rition.backend.configure;

import org.springframework.boot.SpringBootConfiguration;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurationSupport;
import rition.backend.api.interceptor.ResponseIdInterceptor;
import rition.backend.api.resolver.RequestIdArgumentResolver;

import java.util.List;

@Configuration
@SpringBootConfiguration
public class WebAppConfig extends WebMvcConfigurationSupport {

    private final RequestIdArgumentResolver requestIdArgumentResolver;
    private final ResponseIdInterceptor requestIdInterceptor;

    public WebAppConfig(RequestIdArgumentResolver requestIdArgumentResolver,
                        ResponseIdInterceptor requestIdInterceptor) {
        this.requestIdArgumentResolver = requestIdArgumentResolver;
        this.requestIdInterceptor = requestIdInterceptor;
    }

    @Override
    protected void addArgumentResolvers(List<HandlerMethodArgumentResolver> argumentResolvers) {
        argumentResolvers.add(requestIdArgumentResolver);
    }

    @Override
    protected void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(requestIdInterceptor)
                .addPathPatterns("/**");
    }
}
