package rition.common.data.dto.log;

import com.fasterxml.jackson.annotation.JsonInclude;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.Duration;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@JsonInclude(JsonInclude.Include.NON_NULL)
public class RequestProcessTraceRecord {
    private String requestId;

    private OperationStatus status;

    private String message;

    private Exception exception;

    private long cost;

    private Object extra;

    public static RequestProcessTraceRecord success(String requestId, String message, Object extraData) {
        return RequestProcessTraceRecord.builder()
                .requestId(requestId)
                .message(message)
                .extra(extraData)
                .status(OperationStatus.OK)
                .build();
    }

    public static RequestProcessTraceRecord success(String requestId,
                                                    String message,
                                                    long cost,
                                                    Object extraData) {
        return RequestProcessTraceRecord.builder()
                .requestId(requestId)
                .message(message)
                .extra(extraData)
                .cost(cost)
                .status(OperationStatus.OK)
                .build();
    }

    public enum OperationStatus {
        /**
         * 成功
         */
        OK,

        /**
         * 失败
         */
        FAIL,

        /**
         * 部分完成
         */
        PARTIALLY_FINISHED
    }
}
