package rition.common.exception;


import rition.common.exception.code.ServiceCode;

import java.util.Objects;

public class ServiceException extends RuntimeException {
    private final int code;

    public ServiceException(ServiceCode code) {
        this(code, code.getMsg());
    }

    public ServiceException(ServiceCode code, String message) {
        super(message);
        this.code = code.getCode();
    }

    public ServiceException(int code) {
        this(Objects.requireNonNull(ServiceCode.valueOf(code)));
    }

    public ServiceException(int code, String message) {
        super(message);
        this.code = code;
    }

    public ServiceCode getCode() {
        return ServiceCode.valueOf(code);
    }

    public int getCodeValue() {
        return code;
    }

    /**
     * 直接按错误码抛出异常
     *
     * @param code 错误码
     * @throws ServiceException .
     */
    public static void error(int code) throws ServiceException {
        throw new ServiceException(code);
    }
}
